@@ -58,6 +58,9 @@ class Physics:
         self.futureCTrav = CollisionTraverser("collision traverser for future position tests")
         if self.getConfig("show_collisions"):
             self.futureCTrav.showCollisions(render)
+        self.stepCTrav = CollisionTraverser("collision traverser step up detection")
+        if self.getConfig("show_collisions"):
+            self.stepCTrav.showCollisions(render)
         self.physics_pusher = PhysicsCollisionHandler()
         self.physics_pusher.addInPattern('%fn-in')
         self.physics_pusher.addOutPattern('%fn-out')
@@ -70,6 +73,8 @@ class Physics:
         self.collisionevent_handler.addInPattern('%fn-in')
         self.collisionevent_handler.addOutPattern('%fn-out')
 
+        self.char_collision_queue_handler = CollisionHandlerQueue()
+
         self.event_mask = BitMask32(0x80)  #1000 0000
         self.body_mask = BitMask32(0x70)  #0111 0000
         self.ray_mask = BitMask32(0x0f)  #0000 1111
@@ -91,18 +96,26 @@ class Physics:
         self.ray_ids = []
         self.ignore_ray_cycle = []
         point_a = Point3(0, 0, self.getConfig("player_height")/1.8)
-        point_b = Point3(0, 0, -self.getConfig("stepheight"))
+        point_b = Point3(0, 0, -self.getConfig("stepheight_down"))
         self.foot_ray_id = "foot_ray_check"
         self.registerRayCheck(self.foot_ray_id, point_a, point_b, self.main_node, True)
 
-        self.placed = False
+        '''
+        # Enable for debugging positions
+        # just place whenever and wherever needed
         self.placer_a = loader.loadModel("models/zup-axis")
         self.placer_a.setScale(0.05)
+        self.placer_a.setColorScale(1,0,0,1)
         self.placer_a.reparentTo(self.main_node)
         self.placer_b = loader.loadModel("models/zup-axis")
         self.placer_b.reparentTo(render)
         self.placer_b.setScale(0.05)
-
+        self.placer_b.setColorScale(0,1,0,1)
+        self.placer_c = loader.loadModel("models/zup-axis")
+        self.placer_c.reparentTo(render)
+        self.placer_c.setScale(0.05)
+        self.placer_c.setColorScale(0,0,1,1)
+        '''
 
     def startPhysics(self):
         """Start and set up the remaining physics parts of the character
@@ -123,7 +136,7 @@ class Physics:
         zB = self.getConfig("player_height") - r
         # actual setup of the characters body collision spheres
         self.charCollisions.node().addSolid(CollisionSphere(0, 0, zA, r))
-        #self.charCollisions.node().addSolid(CollisionSphere(0, 0, zB, r))
+        self.charCollisions.node().addSolid(CollisionSphere(0, 0, zB, r))
 
         self.charCollisions.node().setIntoCollideMask(self.body_mask)
         self.charCollisions.node().setFromCollideMask(self.body_mask)
@@ -132,15 +145,17 @@ class Physics:
             self.charCollisions.show()
         self.physics_pusher.addCollider(self.charCollisions, self.main_node)
         base.cTrav.addCollider(self.charCollisions, self.physics_pusher)
+        self.stepCTrav.addCollider(self.charCollisions, self.char_collision_queue_handler)
 
         # Create the big sphere around the caracter which can be used for special events
         self.eventCollider = self.main_node.attachNewNode(CollisionNode(self.getConfig("char_collision_name")))
-        #self.eventCollider.node().addSolid(CollisionSphere(0, 0, self.getConfig("player_height")/2.0, self.getConfig("player_height")/2.0))
+        self.eventCollider.node().addSolid(CollisionSphere(0, 0, self.getConfig("player_height")/2.0, self.getConfig("player_height")/2.0))
         self.eventCollider.node().setIntoCollideMask(self.event_mask)
         self.eventCollider.node().setFromCollideMask(self.event_mask)
         if self.getConfig("show_collisions"):
             self.eventCollider.show()
-        base.cTrav.addCollider(self.eventCollider, self.collisionevent_handler)
+        if self.getConfig("event_collision_enabled"):
+            base.cTrav.addCollider(self.eventCollider, self.collisionevent_handler)
 
         self.accept("charBody-in", self.checkInBodyContact)
         self.accept("charBody-out", self.checkOutBodyContact)
@@ -426,6 +441,26 @@ class Physics:
 
             shiftZ = 0
 
+            if self.getConfig("do_step_up_check"):
+                #
+                # In this section we are going to check if the character should
+                # move up on stairs. For this we need to gather all collision
+                # points that have occured with the characters body collisions
+                #
+                self.stepCTrav.traverse(render)
+                entries = self.char_collision_queue_handler.getEntries()
+                for collision in entries:
+                    if collision.hasSurfacePoint():
+                        newPos = collision.getSurfacePoint(self)
+                        # chec if the found collision is within the range of a step
+                        if newPos.getZ() >= self.getConfig("stepheight_min_up") and newPos.getZ() <= self.getConfig("stepheight_max_up"):
+                            # move up a tiny bit, so we won't stuck in the ground
+                            newPos.setZ(newPos.getZ() + 0.2)
+                            # also move forward by the given amount so we won't fall off of the step right away
+                            newPos.setY(newPos.getY() - self.getConfig("step_up_forward_distance"))
+                            self.main_node.setFluidPos(self.main_node, newPos)
+                            return True
+
             if char_step_collision is None:
                 # check for additianal collisions with the char sphere or
                 # any other solids that serve as "stand on ground" check
@@ -456,9 +491,6 @@ class Physics:
                             moveVec.setZ(0)
                             moveVec *= -1
                             moveVec *= dt
-                            if not self.placed:
-                                self.placer_b.setPos(posR)
-                                self.placed = True
                             self.main_node.setFluidPos(self.main_node, moveVec)
                             self.toggleFlyMode(False)
                             return False
@@ -617,47 +649,63 @@ class Physics:
         with the upward speed given in jumpForce and forward given in speed.
         Note, if the actorNode shouldn't slide after landing call the
         physics.land function with the same actorNode"""
+        #
+        # Reset stuff
+        #
         # as we leave the ground set the active platform, if any, to None
         self.setActivePlatform(None)
 
+        # make sure we aren't in fly mode, otherwise we can't jump at all
+        self.toggleFlyMode(False)
+
+        #
+        # Jump vector calculation
+        #
         dt = globalClock.getDt()
         jumpVec = Vec3(
             jump_direction.getX()*dt,
             -((forwardSpeed*self.getConfig("jump_forward_force_mult"))+jump_direction.getY())*dt,
-            (self.getConfig("phys_jump_strength")+jump_direction.getZ())*dt)
+            (self.getConfig("phys_jump_strength")+jump_direction.getZ()))#*dt)
         jumpVec *= self.getConfig("jump_strength")
 
-        # rotate the extraSpeedVector to face the same direction the main_node vector
-        charVec = self.main_node.getRelativeVector(render, extraSpeedVec)
-        charVec.normalize()
-        rotatedExtraSpeedVec = charVec * extraSpeedVec.length()
-
-        jumpVec += rotatedExtraSpeedVec*dt
+        #
+        # Extra speed calculation
+        #
+        # rotate the extraSpeedVector to face the same direction the main_node
+        # does and add it to the jump vector
+        jumpVec += self.main_node.getRelativeVector(render, extraSpeedVec)
 
+        #
+        # Push the character
+        #
+        # now add the actual force to the characters physic node
         self.actorNode.getPhysicsObject().addLocalImpulse(jumpVec)
 
+        #
+        # Velocity checks
+        #
         vel = self.actorNode.getPhysicsObject().getVelocity()
         velX = vel.getX()
         velY = vel.getY()
         velZ = vel.getZ()
 
         # Make sure we don't jump/move faster than we are alowed to
-        #if abs(velX) > self.getConfig("max_jump_force_internal_X") \
-        #or abs(velY) > self.getConfig("max_jump_force_internal_Y"):
-        #    # we need to make sure X and Y are at the same distance
-        #    # as before otherwise jump direction will be shifted
-        #    if abs(velX) > abs(velY):
-        #        pass
-        #        #TODO: Calculate diff between x and y and subtract/add to respective other
-        #    if velX < 0:
-        #        velX = -self.getConfig("max_jump_force_internal_X")
-        #    else:
-        #        velX = self.getConfig("max_jump_force_internal_X")
-        #if abs(velY) > self.getConfig("max_jump_force_internal_Y"):
-        #    if velY < 0:
-        #        velY = -self.getConfig("max_jump_force_internal_Y")
-        #    else:
-        #        velY = self.getConfig("max_jump_force_internal_Y")
+        if abs(velX) > self.getConfig("max_jump_force_internal_X") \
+        or abs(velY) > self.getConfig("max_jump_force_internal_Y"):
+            # we need to make sure X and Y are at the same distance
+            # as before otherwise jump direction will be shifted
+            if abs(velX) > abs(velY):
+                pass
+                #TODO: Calculate diff between x and y and subtract/add to respective other
+            if velX < 0:
+                velX = -self.getConfig("max_jump_force_internal_X")
+            else:
+                velX = self.getConfig("max_jump_force_internal_X")
+        if abs(velY) > self.getConfig("max_jump_force_internal_Y"):
+            if velY < 0:
+                velY = -self.getConfig("max_jump_force_internal_Y")
+            else:
+                velY = self.getConfig("max_jump_force_internal_Y")
         if abs(velZ) > self.getConfig("max_jump_force_internal_Z"):
             if velZ < 0:
                 velZ = -self.getConfig("max_jump_force_internal_Z")
@@ -667,6 +715,7 @@ class Physics:
         self.actorNode.getPhysicsObject().setVelocity(velX, velY, velZ)
 
     def land(self):
+        """Reset velocities of the characters physic node"""
         self.actorNode.getPhysicsObject().setVelocity(0,0,0)
 
     def setActivePlatform(self, platform):
