@@ -35,6 +35,7 @@ class Plugin:
         self.core = core
         self.do_ledge_grab = False
         self.request_idle = False
+        self.canInitiateGrab = True
 
         #
         # SETUP STATES
@@ -46,10 +47,11 @@ class Plugin:
             self.STATE_LEDGE_GRAB_RIGHT,
             self.STATE_LEDGE_GRAB_UP]
 
-        # register the wall run states
+        # register the ledge grab states
         self.core.plugin_registerState(
             self.STATE_LEDGE_GRAB,[
                 self.core.STATE_FALL,
+                self.core.STATE_JUMP,
                 self.STATE_LEDGE_GRAB_UP,
                 self.STATE_LEDGE_GRAB_LEFT,
                 self.STATE_LEDGE_GRAB_RIGHT]
@@ -61,7 +63,8 @@ class Plugin:
             self.STATE_LEDGE_GRAB_UP,
             ["*"],
             isFlying=True,
-            isPreventRotation=True)
+            isPreventRotation=True,
+            isPreventJump=True)
         self.core.plugin_registerState(
             self.STATE_LEDGE_GRAB_LEFT,[
                 self.core.STATE_FALL]
@@ -144,7 +147,11 @@ class Plugin:
         # "preload" all animations of the character
         self.core.bindAllAnims()
 
+    def __resetCanInitiateGrab(self, task):
+        self.canInitiateGrab = True
+
     def action(self, intel_action):
+        if not self.core.getConfig("ledge_grab_enabled"): return
         #
         # LEDGE GRAB LOGIC
         #
@@ -152,6 +159,12 @@ class Plugin:
         if self.request_idle:
             # check for the ledge grab up animation state
             ac = self.core.getAnimControl(self.LEDGE_GRAB_UP)
+
+            if not taskMgr.hasTaskNamed("resetCanGrab"):
+                # reset the can initialize grab after a short while
+                # This should be done whenever we leave ledge grab
+                taskMgr.doMethodLater(0.25, self.__resetCanInitiateGrab, "resetCanGrab")
+
             if ac.isPlaying():
                 # as long as it's playing we won't transition anywhere
                 # and do as we're still in normal ledge grab mode
@@ -292,10 +305,12 @@ class Plugin:
 
         elif ledge_collision is not None \
         and intel_action \
-        and self.core.state not in self.ledge_grab_states:
+        and self.core.state not in self.ledge_grab_states \
+        and self.canInitiateGrab:
             #
             # INITIATE LEDGE GRAB
             #
+            self.canInitiateGrab = False
             # only initiate a ledge grab if certain conditions are met
             ledge_normal = None
             if self.core.hasSurfaceNormal(ledge_collision):
@@ -311,10 +326,17 @@ class Plugin:
                     self.do_ledge_grab = True
         if self.do_ledge_grab:
             self.core.toggleFlyMode(True)
+            self.core.resetAfterJump()
             self.core.rotation = None
             vec = self.core.plugin_getMoveDirection()
             vec.setY(0)
             self.core.plugin_setMoveDirection(vec)
+        else:
+            if not taskMgr.hasTaskNamed("resetCanGrab"):
+                # reset the can initialize grab after a short while
+                # This should be done whenever we leave ledge grab
+                taskMgr.doMethodLater(0.25, self.__resetCanInitiateGrab, "resetCanGrab")
+
         return self.do_ledge_grab
 
     def useStamina(self):
